




/*
 *
 *          Copyright (C) 1994, M. A. Sridhar
 *  
 *
 *     This software is Copyright M. A. Sridhar, 1994. You are free
 *     to copy, modify or distribute this software  as you see fit,
 *     and to use  it  for  any  purpose, provided   this copyright
 *     notice and the following   disclaimer are included  with all
 *     copies.
 *
 *                        DISCLAIMER
 *
 *     The author makes no warranties, either expressed or implied,
 *     with respect  to  this  software, its  quality, performance,
 *     merchantability, or fitness for any particular purpose. This
 *     software is distributed  AS IS.  The  user of this  software
 *     assumes all risks  as to its quality  and performance. In no
 *     event shall the author be liable for any direct, indirect or
 *     consequential damages, even if the  author has been  advised
 *     as to the possibility of such damages.
 *
 */


#include "rectangl.h"
#include "arc.h"
#include "applic.h"
#include "cntroler.h"
#include "dwgsurf.h"
#include "lineseg.h"
#include "ellipse.h"
#include "chord.h"

#include <math.h>
#include <stdlib.h>

UI_Chord::UI_Chord (const UI_Rectangle &boundingRect, long startAngleDeg64,
                    long subtendedDeg64):
                    _arc (boundingRect, startAngleDeg64, subtendedDeg64)
{
    _boundingEllipse = boundingRect;
    _startAngle      = startAngleDeg64;
    _subtAngle       = subtendedDeg64;
}



UI_Chord::UI_Chord (const UI_Point &p1, const UI_Point &p2,
                    const UI_Point& origin): _arc (p1, p2, origin)
{
    _boundingEllipse = _arc.Ellipse().BoundingRectangle();
    _startAngle      = _arc.StartAngle();
    _subtAngle       = _arc.SubtendedAngle();
}



bool UI_Chord::DrawOn (UI_DrawingSurface& sfc, const UI_Point& p) const
{
    if (p != UI_Point (0, 0)) {
        UI_Chord a (_boundingEllipse + p, _startAngle, _subtAngle);
        sfc.DrawChord (a, UID_Outline);
    }
    else
        sfc.DrawChord (*this, UID_Outline);
    return TRUE;
}



bool UI_Chord::Fill (UI_DrawingSurface& sfc) const
{
    sfc.DrawChord (*this, UID_Fill);
    return TRUE;
}



bool UI_Chord::ReshapeTo (const UI_Point &p1, const UI_Point &p2)
{
    _arc.ReshapeTo (p1, p2);
    _startAngle = _arc.StartAngle();
    _subtAngle  = _arc.SubtendedAngle();
    _boundingEllipse = _arc.Ellipse().BoundingRectangle();
    return TRUE;
}



void UI_Chord::StartAngle (long startAngle64)
{
    _startAngle = startAngle64;
    _arc.StartAngle (_startAngle);
}



void UI_Chord::SubtendedAngle (long subtAngle64)
{
    _subtAngle   = subtAngle64;
    _arc.SubtendedAngle (_subtAngle);
}



void UI_Chord::Origin (const UI_Point &origin)
{
    _boundingEllipse.Origin (origin);
    _arc.Ellipse().BoundingRectangle().Origin (origin);
}




UI_Ellipse UI_Chord::Ellipse() const
{
    return UI_Ellipse (_boundingEllipse);
}



UI_Rectangle UI_Chord::BoundingRectangle() const
{
    UI_Point pa, pb;

    struct UI_Arc::UI_PointPair p = EndPoints ();
    return (*(new UI_Rectangle (p.p1, 10, 10))); // ????? HUH ????
}



struct UI_Arc::UI_PointPair UI_Chord::EndPoints () const
{
    return _arc.EndPoints();
}


bool UI_Chord::IntersectsBoundary (const UI_Rectangle& r) const
{
    long topx = r.Left();
    long topy = r.Top();
    long a    = r.Width();
    long b    = r.Height();
    struct UI_Arc::UI_PointPair pp = EndPoints();
    UI_Point pa = pp.p1;
    UI_Point pb = pp.p2;

    if (_arc.IntersectsBoundary (r) == TRUE) return TRUE;

    UI_LineSegment la (UI_Point (topx, topy), UI_Point (topx+a, topy));
    UI_LineSegment lb (UI_Point (topx, topy), UI_Point (topx, topy+b));
    UI_LineSegment lc (UI_Point (topx, topy+b), UI_Point (topx+a, topy+b));
    UI_LineSegment ld (UI_Point (topx+a, topy), UI_Point (topx+a, topy+b));
    UI_LineSegment cl (pb, pa);

    return (la.Intersects (cl) || lb.Intersects (cl) || lc.Intersects (cl) ||
            ld.Intersects (cl));
}

